""" Custom error definitions for data-fetcher """

import logging
import sys
import traceback
from enum import Enum

_LOG = logging.getLogger(__name__)

# pylint: disable=W1202


class Errors(Enum):
    """
    Assorted constants and functions involving errors

    """

    NO_PROFILE = 1
    MISSING_SETTING = 2
    LOCATION_SERVICE_TIMEOUT = 3
    LOCATION_SERVICE_REDIRECTS = 4
    LOCATION_SERVICE_ERROR = 5
    NO_LOCATOR = 6
    FILE_ERROR = 7
    NGAS_SERVICE_TIMEOUT = 8
    NGAS_SERVICE_REDIRECTS = 9
    NGAS_SERVICE_ERROR = 10
    SIZE_MISMATCH = 11
    FILE_EXISTS_ERROR = 12
    FILE_NOT_FOUND_ERROR = 13


class NoProfileException(Exception):
    """ throw this if Capo profile can't be determined"""


class MissingSettingsException(Exception):
    """ throw this if a required setting isn't found in command-line args """


class LocationServiceTimeoutException(Exception):
    """ throw this if the location service takes too long to return """


class LocationServiceRedirectsException(Exception):
    """ throw this if the location service
        complains about too many redirects
    """


class LocationServiceErrorException(Exception):
    """ throw this if the location service falls over """


class NGASServiceTimeoutException(Exception):
    """ throw this if the NGAS service time out """


class NGASServiceRedirectsException(Exception):
    """ throw this if the NGAS service complains about too many redirects """


class NGASServiceErrorException(Exception):
    """ throw this if the NGAS service falls over """


class NoLocatorException(Exception):
    """ throw this if no product locator could be determined from
        command-line arguments

    """


class FileErrorException(Exception):
    """ throw this if file retriever can't access or create the output
        directory

    """


class SizeMismatchException(Exception):
    """ throw this if the size of the file retrieved
        doesn't match expected size

    """


TERMINAL_ERRORS = {
    Errors.NO_PROFILE: 'no CAPO profile provided',
    Errors.MISSING_SETTING: 'missing required setting',
    Errors.LOCATION_SERVICE_TIMEOUT: 'request to locator service timed out',
    Errors.LOCATION_SERVICE_REDIRECTS: 'too many redirects on locator service',
    Errors.LOCATION_SERVICE_ERROR: 'catastrophic error on locator service',
    Errors.NO_LOCATOR: 'product locator not found',
    Errors.FILE_ERROR: 'not able to open specified location file',
    Errors.FILE_EXISTS_ERROR: 'specified location file exists',
    Errors.NGAS_SERVICE_TIMEOUT: 'request to NGAS timed out',
    Errors.NGAS_SERVICE_REDIRECTS: 'too many redirects on NGAS service',
    Errors.NGAS_SERVICE_ERROR: 'catastrophic error on NGAS service',
    Errors.SIZE_MISMATCH: 'retrieved file not expected size',
    Errors.FILE_NOT_FOUND_ERROR: 'target directory or file not found'
}


def get_error_descriptions():
    """ user-friendly reporting of errors """

    result = 'Return Codes:\n'
    for error in Errors:
        result += '\t{}: {}\n'.format(error.value, TERMINAL_ERRORS[error])
    return result


def terminal_error(errno):
    """ report error, then throw in the towel
    """
    if errno in TERMINAL_ERRORS:
        _LOG.error(TERMINAL_ERRORS[errno])
    else:
        _LOG.error('unspecified error {}'.format(errno))

    sys.exit(errno.value)


def exception_to_error(exception):
    """ translate an exception to one of our custom errors
    """
    switcher = {
        'NoProfileException': Errors.NO_PROFILE,
        'MissingSettingsException': Errors.MISSING_SETTING,
        'LocationServiceTimeoutException': Errors.LOCATION_SERVICE_TIMEOUT,
        'LocationServiceRedirectsException': Errors.LOCATION_SERVICE_REDIRECTS,
        'LocationServiceErrorException': Errors.LOCATION_SERVICE_ERROR,
        'NoLocatorException': Errors.NO_LOCATOR,
        'FileErrorException': Errors.FILE_ERROR,
        'FileExistsError': Errors.FILE_EXISTS_ERROR,
        'NGASServiceTimeoutException': Errors.NGAS_SERVICE_TIMEOUT,
        'NGASServiceRedirectsException': Errors.NGAS_SERVICE_REDIRECTS,
        'NGASServiceErrorException': Errors.NGAS_SERVICE_ERROR,
        'SizeMismatchException': Errors.SIZE_MISMATCH,
        'FileNotFoundError': Errors.FILE_NOT_FOUND_ERROR,
    }
    return switcher.get(exception.__class__.__name__)


def terminal_exception(exception):
    """ Report this exception, then throw in the towel.
        should be used by DataFetcher -ONLY-
    """
    errorno = exception_to_error(exception)
    _LOG.debug(traceback.format_exc())
    _LOG.error(str(exception))
    sys.exit(errorno.value)
